unit Unit1;
{
NetValue (C) 2008 P. Walker
Contact as philboard at http://www.freepcb.com/

A program to modify a TinyCAD(c) netlist into a format
suitable to be imported into FreePCB(c) containing extra
'value' string information.

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any
later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program; if not, write to the
Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Menus, Printers;

type
  PartListItem = ^PartItem;
  AttListItem = ^AttItem;
  MyNetItem = ^NetItem;
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    File1: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    Print1: TMenuItem;
    N2: TMenuItem;
    Save1: TMenuItem;
    Open1: TMenuItem;
    New1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Contents1: TMenuItem;
    Panel1: TPanel;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    lsbAttAvail: TListBox;
    lsbAttSel: TListBox;
    btnAttSel: TButton;
    btnAttUnsel: TButton;
    UpDown1: TUpDown;
    btnProcess: TButton;
    GroupBox4: TGroupBox;
    Memo1: TMemo;
    pnlHelp: TPanel;
    Memo2: TMemo;
    btnCloseHelp: TButton;
    PrintDialog1: TPrintDialog;
    FontDialog1: TFontDialog;
    menuFont: TMenuItem;
    PrinterSetupDialog1: TPrinterSetupDialog;
    menuPSetup: TMenuItem;
    License1: TMenuItem;
    gpbLicense: TGroupBox;
    Memo3: TMemo;
    btnCloseLicense: TButton;
    procedure btnProcessClick(Sender: TObject);
    procedure btnAttSelClick(Sender: TObject);
    procedure btnAttUnselClick(Sender: TObject);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure Open1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Contents1Click(Sender: TObject);
    procedure btnCloseHelpClick(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure menuFontClick(Sender: TObject);
    procedure menuPSetupClick(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure License1Click(Sender: TObject);
    procedure btnCloseLicenseClick(Sender: TObject);
  private
    { Private declarations }
    function  XtractStr(Const S: string; From: integer; Delim: char): string;
    procedure AttNameAdd(S: string);
    function  StringReplace(sSrce, sSrch, sRepl: string): string;
  public
    { Public declarations }
  end;

  PartItem = record
    RefDes: string;
    Footprint: string;
    AttPtr: AttListItem;
  end;


  AttItem = record
    AttName: string;
    AttValue: string;
    Next: AttListItem;
  end;

  NetItem = record
    Name: string;
    Value: string;
  end;
var
  Form1: TForm1;
  xRefDir, xRefFile, schDir, schFile: string;
  LParts: TList;
  LNets: TList;
  valDir, valFile, netDir, netFile: string;

implementation

{$R *.DFM}

procedure TForm1.AttNameAdd(S: string);
begin
  if (lsbAttAvail.Items.IndexOf(S)<0) then  lsbAttAvail.Items.Add(S);
end;

function TForm1.StringReplace(sSrce, sSrch, sRepl: string): string;
var nPos, nLookFor: integer;
begin
  nPos:= Pos(sSrch, sSrce);
  nLookFor:= Length(sSrch);
  while (nPos > 0) do
  begin
    Delete(sSrce, nPos, nLookFor);
    Insert(sRepl, sSrce, nPos);
    nPos:= Pos(sSrch, sSrce);
  end;
  StringReplace:= sSrce;
end;

procedure TForm1.btnProcessClick(Sender: TObject);
var sTemp, sTemp2, sSep, sat: string;
    nPtr1,nPtr2: integer;
    PPart: PartListItem;
    APart: AttListItem;
    NNet: MyNetItem;

begin
  nPtr1:=0;
  Memo1.Clear;
  btnCloseHelpClick(Self);
  //Parts header
  Memo1.Lines.Add('*PADS-PCB*');
  Memo1.Lines.Add('*PART*');

  //Process parts and attributes
  while (nPtr1 < LParts.Count) do
  begin
    sTemp:= '';
    sSep:=  '' ;
    sat:= '';
    PPart:= LParts.Items[nPtr1];
    nPtr2:= 0;
    while (nPtr2< lsbAttSel.Items.Count) do
    begin
      APart:= PPart.AttPtr;
      sTemp2:= sSep + '???';
      while (APart <> nil) do
      begin
        if (lsbAttSel.Items.Strings[nPtr2] = APart.AttName) then
          sTemp2:= sSep + APart.AttValue ;
        sat:= ' @'; //Add space for readability
        APart:=APart.Next;
      end;
      sSep:= ', ';
      sTemp:=  sTemp + sTemp2;
      nPtr2:= nPtr2+1;
    end;
    sTemp:= PPart.RefDes + char(9) + sTemp + sat + PPart.Footprint;
    memo1.Lines.Add(sTemp);
    nPtr1:= nPtr1+1;
  end;
   memo1.Lines.Add( ' ' );

  // Net header
  memo1.Lines.Add( '*NET*' );

  // Process nets and pins here
  for nPtr1:= 0 to LNets.Count-1 do
  begin
    NNet:= LNets.Items[nPtr1];
    memo1.Lines.Add('*SIGNAL*  ' + NNet.Name);

    sTemp:= StringReplace(NNet.Value, '),(', ' ');
    sTemp:= StringReplace(sTemp, '(', '');
    sTemp:= StringReplace(sTemp, ')', '');
    sTemp:= StringReplace(sTemp, ',', '.');
                                    
    memo1.Lines.Add(sTemp);
  end;

  //Wind up here
    memo1.Lines.Add('*END*');

end;

function TForm1.XtractStr(Const S: string; From: integer; Delim: char): string;
var nPtr: integer;
begin
  nPtr:= From;
  while (nptr < length(S)) do
    if (S[nptr] = Delim) then break
    else nptr:= nPtr + 1;
  From:= nptr + 1;
  nPtr:= From;
  while (nptr < length(S)) do
    if (S[nptr] = Delim) then break
    else nptr:= nPtr + 1;
  XtractStr:= copy(S, From, (nPtr-From));
end;
procedure TForm1.btnAttSelClick(Sender: TObject);
var nTemp: integer;
  sTemp:string;
begin
  nTemp:= lsbAttAvail.ItemIndex;
  if (nTemp < 0) then
    Application.MessageBox('Nothing selected','Select error', MB_OK)
    else begin
      sTemp:= lsbAttAvail.Items.Strings[nTemp];
      lsbAttAvail.Items.Delete(nTemp) ;
      lsbAttSel.Items.Add(sTemp);
      nTemp:= lsbAttSel.Items.Count -1;
      lsbAttSel.ItemIndex:= nTemp;
    end;
end;


procedure TForm1.btnAttUnselClick(Sender: TObject);
var nTemp: integer;
  sTemp:string;
begin
  nTemp:= lsbAttSel.ItemIndex;
  if (nTemp < 0) then
    Application.MessageBox('Nothing selected','Select error', MB_OK)
    else begin
      sTemp:= lsbAttSel.Items.Strings[nTemp];
      lsbAttSel.Items.Delete(nTemp) ;
      lsbAttAvail.Items.Add(sTemp);
    end;
end;

procedure TForm1.UpDown1Click(Sender: TObject; Button: TUDBtnType);
var lTemp, nTemp, mTemp: integer;
begin
  lTemp:= lsbAttSel.Items.Count-1;
  nTemp:= lsbAttSel.ItemIndex;
  mTemp:= nTemp;
  if (nTemp < 0) then begin
    Application.MessageBox('Nothing selected','Select error', MB_OK);
    Exit;
    end;
  if ((Button = btNext) and (nTemp > 0))
    then mTemp:= nTemp - 1;   //Go up
  if ((Button = btPrev) and (nTemp < lTemp))
    then  mTemp:= nTemp + 1;  // Go down
  if (nTemp <> MTemp) then lsbAttSel.Items.Move(nTemp, mTemp); // Move
  lsbAttSel.ItemIndex:= mTemp;

end;

procedure TForm1.Open1Click(Sender: TObject);
var nError: integer;
    F: TextFile;
    S,sName, sValue: string;
    cTemp: char;
    PPart: PartListItem;
    APart,BPart: AttListItem;
    NNet: MyNetItem;
    bFormat: boolean;
begin
  pnlHelp.Visible:= False;
  OpenDialog1.Title:= 'Open TinyCAD Netlist';
  OpenDialog1.Filter := 'TinyCAD Netlist files (*.NET)|*.net';
  OpenDialog1.InitialDir:= schDir;
  OpenDialog1.FileName:= '';
  OpenDialog1.Execute;
  while not FileExists(OpenDialog1.FileName) do
    begin
      nError:= Application.MessageBox('File not found',
        'File Open Error', MB_RETRYCANCEL or MB_ICONEXCLAMATION);
      if (nError =  IDRETRY) then OpenDialog1.Execute else Exit;
    end;
  New1Click(Self);
  schDir:=  ExtractFileDir(OpenDialog1.FileName);
  schFile:= ExtractFileName(OpenDialog1.FileName);
  Form1.Caption:=  'NetValue [' + schFile + ']';
  AssignFile(F, OpenDialog1.FileName);
  Reset (F);
  bFormat:= False;
  lsbAttAvail.Clear;
  lsbAttSel.Clear;
  while not EOF(F) do
    begin
      S:='';
      repeat // Read a line with <lf> and/or <cr> as terminator
        read(F,cTemp);
        if (cTemp = char(10)) then cTemp := char(13);
        S:= S + cTemp;
      until (cTemp = char(13));
      S:= TrimLeft(S);
      // Check for start of symbol instance
      // Create part(refdes, footprint, att_ptr and add to list
      if (pos('COMPONENT',S) =1 ) then
      begin
        New(APart);
        APart.AttName:= 'Symbol';
        APart.AttValue:= Trim(copy(S, pos('= ',S) +2, length(S))) ;
        APart.Next:= nil;
        AttNameAdd(APart.AttName);
        BPart:= APart;

        New(PPart);
        //get string from inside quotes
        PPart.RefDes:= XtractStr(S, 9, char($27));//'Refdes';
        //get string from '= ' to end of line
        PPart.Footprint:= '????';//'Footprint' not yet known;
        PPart.AttPtr:= APart;
        LParts.Add(PPart);
        memo1.Lines.Add('RefDes: '+ PPart.RefDes + ' = ' + PPart.Footprint);
      end;

      // Check for start of attribute instance
      // Create attribute(Name, value, att_ptr) and add to previous part attribute list
     if (pos('OPTION',S)  =1 ) then
     begin
       bFormat:= True;
       sName:= XtractStr(S, 6, char($27));//'AttName';//get string from inside quotes
       sValue:= Trim(copy(S, pos('= ',S) +2, length(S))) ;//'AttValue';//get string from '= ' to end of line
       if (sName = 'Package') then PPart.Footprint:= sValue;

       New(APart);
       APart.AttName:= sName;
       APart.AttValue:= sValue;
       APart.Next:= nil;
       AttNameAdd(APart.AttName);
       memo1.Lines.Add('    Att: '+ APart.AttName +' = ' + APart.AttValue);
       BPart.Next:= APart;
       BPart:= Apart;
     end;

      // Create Net(Name, value) and add to net list
      if (pos('NET',S)  =1 ) then // nMode:= 1; // Flag first entry
      begin
        New(NNet);
        //get string from inside quotes
        NNet.Name:= XtractStr(S, 3, char($27));//'NetName';
        //get string from '= ' to end of line
        NNet.Value:= Trim(copy(S, pos('= ',S) +2, length(S))) ;//'PinLst';
        LNets.Add(NNet);
        memo1.Lines.Add('Net '+ NNet.Name + ' = ' + NNet.Value);
      end;
    end;
    CloseFile(F);
    if not(bFormat) then Application.MessageBox('Not TinyCAD format',
        'File format error', MB_OK);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  LParts:= TList.Create;
  LNets:= TList.Create;
  xRefDir:= GetCurrentDir;
  schDir:=  GetCurrentDir;
  Memo1.Clear;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  New1Click(Self);
  Application.Terminate;
end;

procedure TForm1.Save1Click(Sender: TObject);
var G: TextFile;
    nIndex: integer;
    sTemp: string;
begin
  // If cross reference exists:-
  if not (Memo1.Lines.Count >0) then
    Application.MessageBox('No data to save'
      ,'Process Error',MB_OK)
  else begin
    sTemp:= copy(OpenDialog1.FileName, 1, pos('.',OpenDialog1.FileName)-1);
    SaveDialog1.FileName:= sTemp + '_V.net';

    SaveDialog1.Title:= 'Net File with Value string';
    SaveDialog1.Filter := 'Net (*.net)|*.NET';
    SaveDialog1.InitialDir:= netDir;
    SaveDialog1.Options:=  [ofOverwritePrompt];
    if SaveDialog1.Execute then
      begin // If filename is valid save cross reference file
        netDir:=  ExtractFileDir(SaveDialog1.FileName);
        netFile:= ExtractFileName(SaveDialog1.FileName);

        // Open net file for writing
        AssignFile(G, SaveDialog1.FileName);
        Rewrite(G);  { Output File selected in dialog box }
        nIndex:=0;
        while (nIndex < Memo1.Lines.Count) do
           begin
             writeln(G, Memo1.Lines.Strings[nIndex]);
             nIndex:=nIndex +1;
           end;
        CloseFile(G); // Close net file
      end;
    end;

end;

procedure TForm1.About1Click(Sender: TObject);
begin
  Application.MessageBox('Copyright, P. Walker 2008 (See Help/License)'
    ,'NetValue Maker, v1.0', MB_OK);
end;

procedure TForm1.Contents1Click(Sender: TObject);
begin
  pnlHelp.Visible:= True;
end;

procedure TForm1.btnCloseHelpClick(Sender: TObject);
begin
  pnlHelp.Visible:= False;
end;

procedure TForm1.Print1Click(Sender: TObject);
var
  Line: Integer;
  PrintText: TextFile;   {declares a file variable}
begin
  if PrintDialog1.Execute then
  begin
    AssignPrn(PrintText);   {assigns PrintText to the printer}
    Rewrite(PrintText);     {creates and opens the output file}
    if Memo2.Visible then
    begin
      Printer.Canvas.Font := Memo1.Font;  {assigns user Font settings to the canvas}
      for Line := 0 to Memo2.Lines.Count - 1 do
        Writeln(PrintText, Memo2.Lines[Line]);	{writes the contents of the Memo2 to the printer object}
    end else
    begin
      Printer.Canvas.Font := Memo1.Font;  {assigns Font settings to the canvas}
      for Line := 0 to Memo1.Lines.Count - 1 do
        Writeln(PrintText, Memo1.Lines[Line]);	{writes the contents of the Memo1 to the printer object}
    end;
    CloseFile(PrintText); {Closes the printer variable}
  end;
end;
procedure TForm1.menuFontClick(Sender: TObject);
begin
  FontDialog1.Execute;
  Memo1.Font:= FontDialog1.Font;
  Memo2.Font:= FontDialog1.Font;
end;

procedure TForm1.menuPSetupClick(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
end;

procedure TForm1.New1Click(Sender: TObject);
var
    PPart: PartListItem;
    APart,BPart: AttListItem;
    nPtr: integer;
begin
  Form1.Caption:=  'NetValue';
  Memo1.Clear;
  lsbAttAvail.Clear;
  lsbAttSel.Clear;
  if (LNets.Count >0) then
    for nptr:= 0 to LNets.Count-1 do dispose(LNets.Items[nPtr]);
  LNets.Clear;
  if (LParts.Count >0) then
    for nptr:= 0 to LParts.Count-1 do
    begin
      PPart:= LParts.Items[nPtr];
      Apart:= PPart.AttPtr;
      while (APart <>nil) do
      begin
        BPart:=APart.Next;
        dispose(APart);
        APart:=BPart;
      end;
    end;
  LParts.Clear;
end;

procedure TForm1.License1Click(Sender: TObject);
begin
  gpbLicense.Visible:= True;
end;

procedure TForm1.btnCloseLicenseClick(Sender: TObject);
begin
  gpbLicense.Visible:= False;
end;

end.

